@@ -10,7 +10,7 @@ require HTTP::Request::Common;
 require Crypt::SSLeay;
 require Exporter;
 
-our $VERSION = "1.00";
+our $VERSION = "1.03.1";
 
 our @ISA = qw(Exporter);
 our @EXPORT_OK = ();
@@ -80,9 +80,9 @@ sub login {
 
     return 0 if $self->{_logged_in};
 
-    if ( $self->{_proxy_enable} >= 1 ) {
+    if ( $self->{_proxy_enable} && $self->{_proxy_enable} >= 1 ) {
         $ENV{HTTPS_PROXY} = $self->{_proxy_name};
-        if ( $self->{_proxy_enable} >= 2 ) {    
+        if ( $self->{_proxy_enable} && $self->{_proxy_enable} >= 2 ) {
             $ENV{HTTPS_PROXY_USERNAME} = $self->{_proxy_user};
             $ENV{HTTPS_PROXY_PASSWORD} = $self->{_proxy_pass};
         }
@@ -106,7 +106,7 @@ sub login {
                 $req = HTTP::Request->new( GET => $1 );
                 $req->header( 'Cookie' => $self->{_cookie} );
                 $res = $self->{_ua}->request( $req );
-                if ( $res->content() =~ /frame name=js src=\/gmail(.*?) / ) {
+                 if ( $res->content() =~ /<script src="(.*)"/ ) {
                     update_tokens( $self, $res );
                     if ( $self->{_proxy_enable} ) {
                         if ( $self->{_proxy_enable} >= 1 ) {
@@ -119,7 +119,7 @@ sub login {
                         }
                     }
                     $self->{_logged_in} = 1;
-                    get_page( $self, start => '', search => '', view => '', req_url => $self->{_mail_url} . $1 );                  
+                    $res = get_page( $self, start => '', search => '', view => '', req_url => $self->{_mail_url} . $1 );
                     return( 1 );
                 } else {
                     $self->{_error} = 1;
@@ -199,7 +199,7 @@ sub get_page {
         method  => '',
         req_url => $self->{_mail_url},
         @_, );
-    my ( $res, $req, $req_url );
+    my ( $res, $req, $req_url, @tees );
 
     unless ( check_login( $self ) ) { return };
 
@@ -214,6 +214,16 @@ sub get_page {
         }
     }
 
+    if ( defined( $args{ 't' } ) ) {
+        if ( ref( $args{ 't' } ) eq 'ARRAY' ) {
+            foreach ( @{ $args{ 't' } } ) {
+                push( @tees, 't' );
+                push( @tees, $_ );
+            }
+            delete( $args{ 't' } );
+        }
+    }
+
     $req_url = $args{ 'req_url' };
     delete( $args{ 'req_url' } );
 
@@ -243,8 +253,8 @@ sub get_page {
             Connection   => 'Keep-Alive',
             'Keep-Alive' => 300,
             Cookie       => $self->{_cookie},
-            Content      => [ view => $view, %args ] );
-        if ( $self->{_proxy_enable} >= 2 ) {
+            Content      => [ view => $view, %args, @tees ] );
+        if ( $self->{_proxy_enable} && $self->{_proxy_enable} >= 2 ) {
             $req->proxy_authorization_basic( $self->{_proxy_user}, $self->{_proxy_pass} );
         }
         $res = $self->{_ua}->request( $req );
@@ -255,7 +265,7 @@ sub get_page {
         }
         $req = HTTP::Request->new( GET => $req_url . "$url" );
         $req->header( 'Cookie' => $self->{_cookie} );
-        if ( $self->{_proxy_enable} >= 2 ) {
+        if ( $self->{_proxy_enable} && $self->{_proxy_enable} >= 2 ) {
             $req->proxy_authorization_basic( $self->{_proxy_user}, $self->{_proxy_pass} );
         }
         $res = $self->{_ua}->request( $req );
@@ -331,16 +341,16 @@ sub edit_labels {
 
     my $action;
 
-    if ( $args{ 'action' } eq 'create' ) {
+    if ( uc( $args{ 'action' } ) eq 'CREATE' ) {
         $action = 'cc_';
         $args{ 'new_name' } = '';
-    } elsif ( $args{ 'action' } eq 'delete' ) {
+    } elsif ( uc( $args{ 'action' } ) eq 'DELETE' ) {
         $action = 'dc_';
         $args{ 'new_name' } = '';
-    } elsif ( $args{ 'action' } eq 'remove' ) {
+    } elsif ( uc( $args{ 'action' } ) eq 'REMOVE' ) {
         $action = 'rc_';
         $args{ 'new_name' } = '';
-    } elsif ( $args{ 'action' } eq 'add' ) {
+    } elsif ( uc( $args{ 'action' } ) eq 'ADD' ) {
         $action = 'ac_';
         $args{ 'new_name' } = '';
         unless ( defined( $args{ 'msgid' } ) ) {
@@ -350,10 +360,9 @@ sub edit_labels {
         } else {
             $args{ 't' } = $args{ 'msgid' };
             delete( $args{ 'msgid' } );
-            $args{ 'method' } = 'get';
             $args{ 'search' } = 'all';
         }
-    } elsif ( $args{ 'action' } eq 'rename' ) {
+    } elsif ( uc( $args{ 'action' } ) eq 'RENAME' ) {
         $args{ 'new_name' } = '^' . validate_label( $self, $args{ 'new_name' } );
         if ( $self->{_error} ) {
             return;
@@ -669,6 +678,8 @@ sub send_message {
 sub get_messages {
     my ( $self ) = shift;
     my ( %args ) = (
+        init    => 1,
+        start => 0,
         @_, );
     my ( $res, $req );
 
@@ -718,6 +729,19 @@ sub get_messages {
             $indv_email{ 'attachments' } = extract_fields( $email_line[9] ) if ( $email_line[9] ne '' );
             push ( @emails, \%indv_email );
         }
+        if ( ( @emails == @{ $functions{ 'ts' } }[1] ) && ( @{ $functions{ 'ts' } }[0] != @{ $functions{ 'ts' } }[2] ) ) {
+            my $start = $args{ 'start' };
+            delete( $args{ 'start' } );
+            if ( $args{ 'cat' } ) {
+                $args{ 'label' } = $args{ 'cat' };
+                delete ( $args{ 'cat' } );
+                delete ( $args{ 'search' } );
+            }
+            my $next_page_emails = get_messages( $self, start => ( $start + @emails ), %args );
+            if ( $next_page_emails ) {
+                @emails = ( @emails, @{ $next_page_emails } );
+            }
+        }
         return ( \@emails );
     } else {
         $self->{_error} = 1;
@@ -806,7 +830,7 @@ sub delete_message {
 sub get_indv_email {
     my ( $self ) = shift;
     my ( %args ) = (
-        view   => 'pt',
+        view   => 'cv',
         @_, );
 
     if ( defined( $args{ 'id' } ) && defined( $args{ 'label' } ) ) {
@@ -869,7 +893,7 @@ sub get_indv_email {
                 my @email = @{ extract_fields( $_ ) };
                 $email[2] = remove_quotes( $email[2] );
                 if ( $email[16] ne '' ) {
-                    my @attachments = @{ extract_fields( $email[16] ) };
+                    my @attachments = @{ extract_fields( $email[17] ) };
                     my @files;
                     foreach ( @attachments ) {
                         my @attachment = @{ extract_fields( $_ ) };
@@ -883,11 +907,11 @@ sub get_indv_email {
                     $message{ 'attachments' } = \@files;
                 }
                 $message{ 'id' }      = $email[2];
-                $message{ 'sender' }  = remove_quotes( $email[6] );
-                $message{ 'sent' }    = remove_quotes( $email[8] );
-                $message{ 'to' }      = remove_quotes( $email[9] );
-                $message{ 'read' }    = remove_quotes( $email[13] );
-                $message{ 'subject' } = remove_quotes( $email[14] );
+                $message{ 'sender' }  = remove_quotes( $email[7] );
+                $message{ 'sent' }    = remove_quotes( $email[9] );
+                $message{ 'to' }      = remove_quotes( $email[10] );
+                $message{ 'read' }    = remove_quotes( $email[14] );
+                $message{ 'subject' } = remove_quotes( $email[15] );
                 if ( $args{ 'th' } eq $email[2] ) {
                     foreach ( @{ $functions{ 'mb' } } ) {
                         my $body = extract_fields( $_ );
@@ -911,6 +935,120 @@ sub get_indv_email {
     }
 }
 
+sub get_mime_email {
+    my ( $self ) = shift;
+    my ( %args ) = (
+        view   => 'om',
+        @_, );
+
+    if ( defined( $args{ 'id' } ) && defined( $args{ 'label' } ) ) {
+        $args{ 'label' } = validate_label( $self, $args{ 'label' } );
+        if ( $self->error() ) {
+            return;
+        } else {
+            $args{ 'cat' } = $args{ 'label' };
+            delete( $args{ 'label' } );
+            $args{ 'search' } = 'cat';
+        }
+        $args{ 'th' } = $args{ 'id' };
+        delete( $args{ 'id' } );
+    } elsif ( defined( $args{ 'msg' } ) ) {
+        if ( defined( $args{ 'msg' }->{ 'id' } ) ) {
+            $args{ 'th' } = $args{ 'msg' }->{ 'id' };
+        } else {
+            $self->{_error} = 1;
+            $self->{_err_str} .= "Error: Not a valid msg reference.\n";
+            return;
+        }
+
+        if ( defined( @{ $args{ 'msg' }->{ 'labels' } } ) ) {
+            if ( $args{ 'msg' }->{ 'labels' }->[0] ne '' ) {
+                $args{ 'label' } = validate_label( $self, $args{ 'msg' }->{ 'labels' }->[0] );
+                delete( $args{ 'msg' }->{ 'label' } );
+                if ( $self->error ) {
+                    return;
+                } else {
+                    if ( $args{ 'label' } =~ /^\^.$/ ) {
+                        $args{ 'label' } = cat_to_search( $args{ 'label' } );
+                        $args{ 'search' } = $args{ 'label' };
+                    } else {
+                        $args{ 'cat' } = $args{ 'label' };
+                        $args{ 'search' } = 'cat';
+                    }
+                    delete( $args{ 'label' } );
+                }
+            }
+        }
+        delete( $args{ 'msg' } );
+    } else {
+        $self->{_error} = 1;
+        $self->{_err_str} .= "Error: Must specify either id and label or send a reference to a valid message with msg.\n";
+        return;
+    }
+
+    unless ( check_login( $self ) ) { return };
+
+    my $res = get_page( $self, %args );
+
+    if ( $res->is_success() ) {
+        my $content = $res->content;
+        $content =~ s/\r\n/\n/g;
+        $content =~ s/^(\s*\n)+//;
+        return $content;
+    } else {
+        $self->{_error} = 1;
+        $self->{_err_str} .= "Error: While requesting: '$res->{_request}->{_uri}'.\n";
+        return;
+    }
+}
+
+sub get_contacts {
+    my ( $self ) = shift;
+    my ( %args ) = (
+        @_, );
+    my ( $res, $req );
+
+    $args{ 'view' } = 'cl';
+    $args{ 'search' } = 'contacts';
+    $args{ 'start' } = undef;
+    $args{ 'method' } = 'get';
+    $args{ 'pnl' } = $args{ 'frequent' } ? 'p' : 'a';
+    delete $args{ 'frequent' };
+
+    unless ( check_login( $self ) ) { return };
+
+    $res = get_page( $self, %args );
+
+    if ( $res->is_success() ) {
+        my %functions = %{ parse_page( $self, $res ) };
+
+        if ( $self->{_error} ) {
+            return;
+        }
+        my ( @contacts );
+
+        unless ( defined( $functions{ 'a' } ) ) {
+            return;
+        }
+
+        foreach ( @{ $functions{ 'a' } } ) {
+            my @contact_line = @{ extract_fields( $_ ) };
+            my %indv_contact;
+            $indv_contact{ 'id' }            = remove_quotes( $contact_line[0] );
+            $indv_contact{ 'name1' }         = remove_quotes( $contact_line[1] );
+            $indv_contact{ 'name2' }         = remove_quotes( $contact_line[2] );
+            $indv_contact{ 'email' }         = remove_quotes( $contact_line[3] );
+            $indv_contact{ 'note' }          = remove_quotes( $contact_line[4] );
+            push ( @contacts, \%indv_contact );
+        }
+        return ( \@contacts );
+    } else {
+        $self->{_error} = 1;
+        $self->{_err_str} .= "Error: While requesting: '$res->{_request}->{_uri}'.\n";
+        return;
+    }
+}
+
 sub get_ads {
     my ( $self ) = shift;
     my ( %args ) = (
@@ -922,37 +1060,79 @@ sub get_ads {
 
     unless ( check_login( $self ) ) { return };
 
-    if ( $args{ 'adkey' } ne '' ) {
-        my $res = get_page( $self, %args );
-        if ( $res->is_success() ) {
-            my $ad_text = $res->content();
-            $ad_text =~ s/\n//g;
-            $ad_text =~ /\[\[(.*?)\]\],/;
-            $ad_text = $1;
-            my @indv_ads = @{ extract_fields( $ad_text ) };
-            my @ads;
-            foreach ( @indv_ads ) {
-                my @split_ad = @{ extract_fields( $_ ) };
+    if ( defined( $args{ 'adkey' } ) ) {
+        $args{ 'bb' } = $args{ 'adkey' };
+        delete( $args{ 'adkey' } );
+    } else {
+        $self->{_error} = 1;
+        $self->{_err_str} .= "Error: No addkey provided.\n";
+        return;
+    }
+
+    my $res = get_page( $self, %args );
+    if ( $res->is_success() ) {
+        my $ad_text = $res->content();
+        $ad_text =~ s/\n//g;
+        $ad_text =~ /\[(\[.*?\])\]/;
+        $ad_text = $1;
+        my @indv_ads = @{ extract_fields( $ad_text ) };
+        my @ads;
+        foreach ( @indv_ads ) {
+            my @split_ad = @{ extract_fields( $_ ) };
+            if ( uc( remove_quotes( $split_ad[0] ) ) eq 'A' ) {
+                $split_ad[5] =~ s/<wbr>.*//i;
                 my %ad_hash = (
-                    title       => remove_quotes( $split_ad[0] ),
-                    body        => remove_quotes( $split_ad[1] ),
-                    vendor_link => remove_quotes( $split_ad[2] ),
+                    title       => remove_quotes( $split_ad[2] ),
+                    body        => remove_quotes( $split_ad[3] ),
+                    vendor_link => remove_quotes( $split_ad[5] ),
+                    link        => remove_quotes( $split_ad[4] ), );
+                push( @ads, \%ad_hash );
+            } elsif ( uc( remove_quotes( $split_ad[0] ) ) eq 'RN' ) {
+                if ( $split_ad[3] =~ /redir_url=(.*?)\"/ ) {
+                    my $vendor_link = $1;
+                    my %ad_hash = (
+                        title       => remove_quotes( $split_ad[1] ),
+                        body        => remove_quotes( $split_ad[2] ),
+                        vendor_link => url_unencode( $self, url => $vendor_link ),
+                        link        => remove_quotes( $split_ad[3] ), );
+                    push( @ads, \%ad_hash );
+                }
+            } elsif ( uc( remove_quotes( $split_ad[0] ) ) eq 'RP' ) {
+                my %ad_hash = (
+                    title       => remove_quotes( $split_ad[1] ),
+                    body        => remove_quotes( $split_ad[2] ),
+                    vendor_link => remove_quotes( $split_ad[4] ),
                     link        => remove_quotes( $split_ad[3] ), );
                 push( @ads, \%ad_hash );
             }
-            return( \@ads );
-        } else {
-            $self->{_error} = 1;
-            $self->{_err_str} .= "Error: " . $res->status_line();
         }
+
+        return( \@ads );
     } else {
         $self->{_error} = 1;
-        $self->{_err_str} .= "Error: Must send adkey.\n";
+        $self->{_err_str} .= "Error: " . $res->status_line();
     }
 
     return;
 }
 
+sub url_unencode {
+    my $self = shift;
+    my ( %args ) = (
+        url => '',
+        @_,
+    );
+
+    if ( $args{ 'url' } ) {
+        $args{ 'url' } =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack( "C", hex( $1 ) )/eg;
+        return( $args{ 'url' } );
+    } else {
+        $self->{_error} = 1;
+        $self->{_err_str} .= "Error: Must supply URL to unencode.";
+        return;
+    }
+}
+
 sub get_attachment {
     my ( $self ) = shift;
     my ( %args ) = (
@@ -979,7 +1159,6 @@ sub get_attachment {
             $self->{_err_str} .= "Error: Not a valid attachment.2\n";
             return;
         }
-
         delete( $args{ 'attachment' } );        
     } else {
         $self->{_error} = 1;
@@ -987,7 +1166,6 @@ sub get_attachment {
         return;
     }
 
-    
     unless ( check_login( $self ) ) { return };
     
     my $res = get_page( $self, %args );
@@ -1002,6 +1180,105 @@ sub get_attachment {
     }
 }
 
+sub update_prefs {
+    my ( $self ) = shift;
+    my ( %args ) = (
+        view         => 'tl',
+        act          => 'prefs',
+        search       => 'inbox',
+        @_, );
+
+    unless ( check_login( $self ) ) { return };
+
+    $args{ 'at' } = $self->{_cookies}->{GMAIL_AT};
+
+    my ( %pref_mappings ) = (
+        bx_hs => 'keyboard_shortcuts',
+        ix_nt => 'max_page_size',
+        bx_sc => 'indicators',
+        sx_dn => 'display_name',
+        bx_ns => 'snippets',
+        sx_rt => 'reply_to',
+        sx_sg => 'signature', );
+
+    my ( %pref_args ) = (
+        view    => 'pr',
+        pnl     => 'g',
+        search  => '',
+        start   => '',
+        method  => '',
+    );
+
+    my $pref_res = get_page( $self, %pref_args );
+
+    if ( $pref_res->is_success() ) {
+        my %functions = %{ parse_page( $self, $pref_res ) };
+
+        if ( $self->{_error} ) {
+            return;
+        }
+
+        unless ( defined( $functions{ 'p' } ) ) {
+            return;
+        }
+
+        ### Delete if equal to the string '' ###
+        foreach ( 'signature', 'reply_to', 'display_name' ) {
+            if ( defined( $args{ $_ } ) ) {
+                if ( $args{ $_ } eq '' ) {
+                    $args{ $_ } = '%0A%0D';
+                }
+            }
+        }
+
+        ### Load Prefs if not redefined ###
+        foreach ( @{ $functions{ 'p' } } ) {
+            my ( @setting ) = @{ extract_fields( $_ ) };
+            foreach ( @setting ) {
+                $_ = remove_quotes( $_ );
+            }
+            unless ( defined( $args{ $pref_mappings{ $setting[0] } } ) ) {
+                $args{ 'p_' . $setting[0] } = $setting[1];
+            } else {
+                $args{ 'p_' . $setting[0] } = $args{ $pref_mappings{ $setting[0] } };
+            }
+            delete( $args{ $pref_mappings{ $setting[0] } } );
+        }
+
+        ### Add preferences to be added ###
+        my %rev_pref_mappings;
+        foreach ( keys %pref_mappings ) {
+            $rev_pref_mappings{ $pref_mappings{ $_ } } = $_;
+        }
+        foreach ( keys %args ) {
+            if ( $rev_pref_mappings{ $_ } ) {
+                $args{ 'p_' . $rev_pref_mappings{ $_ } } = $args{ $_ };
+                delete( $args{ $_ } );
+            }
+        }
+
+        my $res = get_page( $self, %args );
+        if ( $res->is_success() ) {
+            my %functions = %{ parse_page( $self, $res ) };
+            if ( @{ $functions{ 'ar' } }[0] == 1 ) {
+                return( 1 );
+            } else {
+                $self->{_error} = 1;
+                $self->{_err_str} .= "Error: While updating user preferences: '" . remove_quotes( @{ $functions{ 'ar' } }[1] ) . "'.\n";
+                return;
+            }
+        } else {
+            $self->{_error} = 1;
+            $self->{_err_str} .= "Error: While updating user preferences: '$res->{_request}->{_uri}'.\n";
+            return;
+        }    
+    } else {
+        $self->{_error} = 1;
+        $self->{_err_str} .= "Error: While requesting user preferences: '$pref_res->{_request}->{_uri}'.\n";
+        return;
+    }
+}
+
 sub recurse_slash {
     my ( $field ) = @_;
     my $count_slashes = 0;
@@ -1088,7 +1365,8 @@ sub extract_fields {
         if ( $end_field ) {
             if ( $field ne '' ) {
                 push ( @fields, $field );
-            }            $field = '';
+            }
+            $field = '';
             $end_field = 0;
         }
     }
@@ -1242,6 +1520,26 @@ On failure, error and error_msg are set.
     #removing a label from a message.
     $gmail->edit_labels( label => 'label_name', action => 'remove', msgid => $message_id );
 
+=head2 UPDATING PREFERENCES
+
+The following are the seven preferences and the allowed values that can currently be changed through Mail::Webmail::Gmail
+
+    keyboard_shortcuts = ( 0, 1 )
+    indicators         = ( 0, 1 )
+    snippets           = ( 0, 1 )
+    max_page_size      = ( 25, 50, 100 )
+    display_name       = ( '', string value up to 96 Characters )
+    reply_to           = ( '', string value up to 320 Characters )
+    signature          = ( '', string value up to 1000 Characters )
+
+Changing preferences can be accomplished by simply sending the preference(s) that you want to change, and the new value.
+
+    $gmail->update_prefs( indicators => 0, reply_to => 'test@test.com' );
+
+To delete display_name, reply_to, or signature simply send a blank string as in the following example.
+
+    $gmail->update_prefs( signature => '' );
+
 =head2 STARRING A MESSAGE
 
 To star or unstar a message use these examples
@@ -1285,8 +1583,8 @@ The Array of hashes is in the following format
     $indv_email{ 'id' }
     $indv_email{ 'new' }
     $indv_email{ 'starred' }
-    $indv_email{ 'date' }
-    $indv_email{ 'sender' }
+    $indv_email{ 'date_received' }
+    $indv_email{ 'sender_email' }
     $indv_email{ 'subject' }
     $indv_email{ 'blurb' }
     @{ $indv_email{ 'labels' } }
@@ -1327,7 +1625,7 @@ returns a Hash of Hashes containing the data from an individual message in the f
 Hash of messages in thread by ID
 
     $indv_email{ 'id' }
-    $indv_email{ 'sender' }
+    $indv_email{ 'sender_email' }
     $indv_email{ 'sent' }
     $indv_email{ 'to' }
     $indv_email{ 'read' }
@@ -1342,6 +1640,37 @@ Hash of messages in thread by ID
         vendor_link => '',
         link        => '', );
 
+=head2 MIME MESSAGES
+
+This will return an individual message in MIME format.
+
+    The parameters to this function are the same as get_indv_email.
+
+    #prints out the MIME format for all messages in the inbox
+    my $messages = $gmail->get_messages( label => $Mail::Webmail::Gmail::FOLDERS{ 'INBOX' } );
+    foreach ( @{ $messages } ) {
+        my $message = $gmail->get_mime_email( msg => $_ );
+        print $message
+    }
+
+    returns a string that contains the MIME formatted email.
+
+=head2 RETRIEVING CONTACTS
+
+The get_contacts method returns a reference to an AoH with all of the
+Contacts.  This can be limited to the 'Frequently Mailed' contacts
+with a flag:
+
+    my $contacts = $gmail->get_contacts( frequent => 1 );
+
+The Array of hashes is in the following format
+
+    $indv_contact{ 'id' }
+    $indv_contact{ 'name1' }
+    $indv_contact{ 'name2' }
+    $indv_contact{ 'email' }
+    $indv_contact{ 'note' }
+
 =head2 SENDING MAIL
 
 The basic format of sending a message is
@@ -1372,6 +1701,18 @@ To permanently delete a message, just send a msgid
 
     $gmail->delete_message( msgid => $msgid );
 
+=head2 ACTING ON MULTIPLE MESSAGES
+
+To act on multiple messages at once send an array ref containing all of the message IDs you with to act on.  This is useful because
+Gmail may temporarily ban you if you send too much traffic in a short amount of time (for instance deleting all of the messaging in
+your SPAM folder one at a time).
+
+    ### Delete Many Messages at once ###
+    $gmail->delete_message( msgid => \@msgids, search => 'spam', del_message => 1 );
+
+    ### Applying a label to multiple messages at once ###
+    $gmail->edit_labels( label => 'label_name', action => 'add', msgid => \@msgids );
+
 =head2 GETTING ATTACHMENTS
 
 There are two ways to get an attachment:
@@ -1449,6 +1790,8 @@ manipulate to extract data from Gmail.
 
 below is a listing of some of the tests that I use as I test various features
 
+SAMPLE USAGE
+
     my ( $gmail ) = Mail::Webmail::Gmail->new( 
             username => 'username', password => 'password', );
 
@@ -1491,6 +1834,29 @@ below is a listing of some of the tests that I use as I test various features
     }
     ###
 
+    ### Delete all SPAM folder messages ###
+    my $messages = $gmail->get_messages( label => $Mail::Webmail::Gmail::FOLDERS{ 'SPAM' } );
+    if ( @{ $messages } ) {
+        my @msgids;
+        foreach ( @{ $messages } ) {
+            push( @msgids, $_->{ 'id' } );
+        }
+        $gmail->delete_message( msgid => \@msgids, search => 'spam', del_message => 1 );
+        if ( $gmail->error() ) {
+            print $gmail->error_msg();
+        } else {
+            print "Deleted " . @msgids . " Messages\n";
+        }
+    }
+    ###
+
+    ### Print out all user defined labels
+    my @labels = $gmail->get_labels();
+    foreach ( @labels ) {
+        print "Label: '" . $_ . "'\n";
+    }
+    ###
+
     ### Prints out new messages attached to the first label
     my @labels = $gmail->get_labels();
 
@@ -1506,7 +1872,7 @@ below is a listing of some of the tests that I use as I test various features
     ###
 
     ### Prints out all attachments
-    $messages = $gmail->get_messages();
+    my $messages = $gmail->get_messages();
 
     foreach ( @{ $messages } ) {
         my $email = $gmail->get_indv_email( msg => $_ );
@@ -1522,27 +1888,27 @@ below is a listing of some of the tests that I use as I test various features
     ###
 
     ### Prints out the vendor link from Ads attached to a message
-    $messages = $gmail->get_messages( label => $Mail::Webmail::Gmail::FOLDERS{ 'INBOX' } );
+    my $messages = $gmail->get_messages( label => $Mail::Webmail::Gmail::FOLDERS{ 'INBOX' } );
 
-    print @{ $messages } . "\n";
     foreach ( @{ $messages } ) {
         print "ID: " . $_->{ 'id' } . "\n";
         my %email = %{ $gmail->get_indv_email( msg => $_ ) };
         if ( $email{ $_->{ 'id' } }->{ 'ads' } ) {
             my $ads;
             foreach $ads ( @{ $email{ $_->{ 'id' } }->{ 'ads' } } ) {
-                print "AD LINK: $ads->{vendor_link}\n";
+                print " - AD LINK: $ads->{vendor_link}\n";
             }
         }
     }
     ###
 
     ### Shows different ways to look through your email
-    $messages = $gmail->get_messages();
+    my $messages = $gmail->get_messages();
 
     print "By folder\n";
-    foreach ( keys %Gmail::FOLDERS ) {
-        my $messages = $gmail->get_messages( label => $Gmail::FOLDERS{ $_ } );
+    foreach ( keys %Mail::Webmail::Gmail::FOLDERS ) {
+        print "KEY: $_\n";
+        my $messages = $gmail->get_messages( label => $Mail::Webmail::Gmail::FOLDERS{ $_ } );
         print "\t$_:\n";
         if ( @{ $messages } ) {
             foreach ( @{ $messages } ) {
@@ -1573,15 +1939,48 @@ below is a listing of some of the tests that I use as I test various features
     }
     ###
 
+    ### Update preferences
+        if ( $gmail->update_prefs( signature => 'Test Sig.', max_page_size => 100 ) ) {
+            print "Preferences Updated.\n";
+        } else {
+            print "Unable to update preferences.\n";
+        }
+    ###
+
+    ### Show all contact email addresses
+    my ( @contacts ) = @{ $gmail->get_contacts() };
+    foreach ( @contacts ) {
+        print $_->{ 'email' } . "\n";
+    }
+    ###
+
+    ### Print out space remaining in mailbox
+    my $remaining = $gmail->size_usage();
+    print "Remaining: '" . $remaining . "'\n";
+    ###
+
 =head1 AUTHOR INFORMATION
 
-Copyright 2004, Allen Holman.  All rights reserved.  
+Copyright 2004-2005, Allen Holman.  All rights reserved.  
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
 
-Address bug reports and comments to: mincus \at cpan \. org.  Or through
-AIM at mincus c03.
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+Address bug reports and comments to: 
+email: mincus \at cpan \. org
+AIM: mincus c03
+Website: http://code.mincus.com
 
 When sending bug reports, please provide the version of Gmail.pm, the version of
 Perl and the name and version of the operating system you are using. 
